@@ -283,7 +283,7 @@ redis_context_handle redisLibeventAttach(struct event_base *base, char *ip0, int
 
   /* Initialize and install read/write events */
   e->rev = event_new(e->base,e->context->c.fd,
-  		     EV_READ,redisLibeventReadEvent,
+  		     EV_READ|EV_PERSIST,redisLibeventReadEvent,
   		     e);
 
   e->wev = event_new(e->base,e->context->c.fd,
@@ -299,11 +299,8 @@ redis_context_handle redisLibeventAttach(struct event_base *base, char *ip0, int
   e->wev_set = 1;
 
   struct bufferevent *pair[2];
-  int opts = BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS;
 
-  opts |= BEV_OPT_THREADSAFE;
-
-  bufferevent_pair_new(base, opts, pair);
+  bufferevent_pair_new(base, TURN_BUFFEREVENTS_OPTIONS, pair);
   e->in_buf = pair[0];
   e->out_buf = pair[1];
   bufferevent_setcb(e->in_buf, receive_message_for_redis, NULL, NULL, e);
