@@ -49,7 +49,7 @@ NULL,
 
 SHATYPE_SHA1, DH_1066, "", DEFAULT_EC_CURVE_NAME, "",
 "turn_server_cert.pem","turn_server_pkey.pem", "", "",
-0,0,0,0,0,
+0,0,0,0,
 #if defined(TURN_NO_TLS)
 1,
 #else
@@ -370,7 +370,8 @@ static char Usage[] = "Usage: turnserver [options]\n"
 " --no-multicast-peers				Disallow peers on well-known broadcast addresses (224.0.0.0 and above, and FFXX:*).\n"
 " -m, --relay-threads		<number>	Number of relay threads to handle the established connections\n"
 "						(in addition to authentication thread and the listener thread).\n"
-"						If set to 0 then application runs in single-threaded mode.\n"
+"						If explicitly set to 0 then application runs in single-threaded mode.\n"
+"						If not set then the default threading policy will be employed.\n"
 "						The default thread number is the number of CPUs.\n"
 "						In older systems (pre-Linux 3.9) the number of UDP relay threads always equals\n"
 "						the number of listening endpoints (unless -m 0 is set).\n"
@@ -461,7 +462,6 @@ static char Usage[] = "Usage: turnserver [options]\n"
 " --dh2066					Use 2066 bits predefined DH TLS key. Default size of the predefined key is 1066.\n"
 " --dh-file	<dh-file-name>			Use custom DH TLS key, stored in PEM format in the file.\n"
 "						Flags --dh566 and --dh2066 are ignored when the DH key is taken from a file.\n"
-" --no-sslv2					Do not allow SSLv2 protocol.\n"
 " --no-sslv3					Do not allow SSLv3 protocol.\n"
 " --no-tlsv1					Do not allow TLSv1 protocol.\n"
 " --no-tlsv1_1					Do not allow TLSv1.1 protocol.\n"
@@ -630,7 +630,7 @@ enum EXTRA_OPTS {
 	DH566_OPT,
 	DH2066_OPT,
 	NE_TYPE_OPT,
-	NO_SSLV2_OPT,
+	NO_SSLV2_OPT, /*deprecated */
 	NO_SSLV3_OPT,
 	NO_TLSV1_OPT,
 	NO_TLSV1_1_OPT,
@@ -737,7 +737,7 @@ static const struct myoption long_options[] = {
 				{ "dh566", optional_argument, NULL, DH566_OPT },
 				{ "dh2066", optional_argument, NULL, DH2066_OPT },
 				{ "ne", required_argument, NULL, NE_TYPE_OPT },
-				{ "no-sslv2", optional_argument, NULL, NO_SSLV2_OPT },
+				{ "no-sslv2", optional_argument, NULL, NO_SSLV2_OPT }, /* deprecated */
 				{ "no-sslv3", optional_argument, NULL, NO_SSLV3_OPT },
 				{ "no-tlsv1", optional_argument, NULL, NO_TLSV1_OPT },
 				{ "no-tlsv1_1", optional_argument, NULL, NO_TLSV1_1_OPT },
@@ -798,7 +798,7 @@ static void set_option(int c, char *value)
 
   switch (c) {
   case NO_SSLV2_OPT:
-	  turn_params.no_sslv2 = get_bool_value(value);
+	  //deprecated
 	  break;
   case NO_SSLV3_OPT:
 	  turn_params.no_sslv3 = get_bool_value(value);
@@ -1791,6 +1791,23 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if(turn_params.external_ip && turn_params.relay_addrs) {
+		size_t ir = 0;
+		for(ir = 0; ir < turn_params.relays_number; ++ir) {
+			if(turn_params.relay_addrs[ir]) {
+				const char* sra = (const char*)turn_params.relay_addrs[ir];
+				if((strstr(sra,"127.0.0.1") != sra)&&(strstr(sra,"::1")!=sra)) {
+					ioa_addr ra;
+					if(make_ioa_addr((const u08bits*)sra,0,&ra)<0) {
+						TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"-X : Wrong address format: %s\n",sra);
+					} else if(ra.ss.sa_family == turn_params.external_ip->ss.sa_family) {
+						ioa_addr_add_mapping(turn_params.external_ip,&ra);
+					}
+				}
+			}
+		}
+	}
+
 	if(turn_params.turn_daemon) {
 #if !defined(TURN_HAS_DAEMON)
 		pid_t pid = fork();
@@ -1861,15 +1878,21 @@ int main(int argc, char **argv)
 
 #if defined(OPENSSL_THREADS)
 
-static pthread_mutex_t* mutex_buf = NULL;
+static char some_buffer[65536];
+
+//array larger than anything that OpenSSL may need:
+static pthread_mutex_t mutex_buf[256];
+static int mutex_buf_initialized = 0;
 
 static void locking_function(int mode, int n, const char *file, int line) {
   UNUSED_ARG(file);
   UNUSED_ARG(line);
-  if (mode & CRYPTO_LOCK)
-    pthread_mutex_lock(&mutex_buf[n]);
-  else
-    pthread_mutex_unlock(&mutex_buf[n]);
+  if(mutex_buf_initialized && (n < CRYPTO_num_locks())) {
+	  if (mode & CRYPTO_LOCK)
+		  pthread_mutex_lock(&(mutex_buf[n]));
+	  else
+		  pthread_mutex_unlock(&(mutex_buf[n]));
+  }
 }
 
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
@@ -1892,12 +1915,13 @@ static int THREAD_setup(void) {
 
 	int i;
 
-	mutex_buf = (pthread_mutex_t*) turn_malloc(CRYPTO_num_locks()
-			* sizeof(pthread_mutex_t));
-	if (!mutex_buf)
-		return 0;
-	for (i = 0; i < CRYPTO_num_locks(); i++)
-		pthread_mutex_init(&mutex_buf[i], NULL);
+	some_buffer[0] = 0;
+
+	for (i = 0; i < CRYPTO_num_locks(); i++) {
+		pthread_mutex_init(&(mutex_buf[i]), NULL);
+	}
+
+	mutex_buf_initialized = 1;
 
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
 	CRYPTO_THREADID_set_callback(id_function);
@@ -1918,7 +1942,7 @@ int THREAD_cleanup(void) {
 
   int i;
 
-  if (!mutex_buf)
+  if (!mutex_buf_initialized)
     return 0;
 
 #if OPENSSL_VERSION_NUMBER >= 0x10000000L
@@ -1928,10 +1952,11 @@ int THREAD_cleanup(void) {
 #endif
 
   CRYPTO_set_locking_callback(NULL);
-  for (i = 0; i < CRYPTO_num_locks(); i++)
-    pthread_mutex_destroy(&mutex_buf[i]);
-  turn_free(mutex_buf,sizeof(pthread_mutex_t));
-  mutex_buf = NULL;
+  for (i = 0; i < CRYPTO_num_locks(); i++) {
+	  pthread_mutex_destroy(&(mutex_buf[i]));
+  }
+
+  mutex_buf_initialized = 0;
 
 #endif
 
@@ -2221,10 +2246,8 @@ static void set_ctx(SSL_CTX* ctx, const char *protocol)
 
 	{
 		int op = 0;
-
 #if defined(SSL_OP_NO_SSLv2)
-		if(turn_params.no_sslv2)
-			op |= SSL_OP_NO_SSLv2;
+		op |= SSL_OP_NO_SSLv2;
 #endif
 		if(turn_params.no_sslv3)
 			op |= SSL_OP_NO_SSLv3;
