@@ -672,7 +672,6 @@ int get_ioa_socket_from_reservation(ioa_engine_handle e, u64bits in_reservation_
   if (e && in_reservation_token && s) {
     *s = rtcp_map_get(e->map_rtcp, in_reservation_token);
     if (*s) {
-      rtcp_map_del_savefd(e->map_rtcp, in_reservation_token);
       return 0;
     }
   }
@@ -681,134 +680,6 @@ int get_ioa_socket_from_reservation(ioa_engine_handle e, u64bits in_reservation_
 
 /* Socket options helpers ==>> */
 
-#define CORRECT_RAW_TTL(ttl) do { if(ttl<0 || ttl>255) ttl=TTL_DEFAULT; } while(0)
-#define CORRECT_RAW_TOS(tos) do { if(tos<0 || tos>255) tos=TOS_DEFAULT; } while(0)
-
-static int get_raw_socket_ttl(evutil_socket_t fd, int family)
-{
-	int ttl = 0;
-
-	if(family == AF_INET6) {
-#if !defined(IPV6_RECVHOPLIMIT)
-		UNUSED_ARG(fd);
-		do { return TTL_IGNORE; } while(0);
-#else
-		socklen_t slen = (socklen_t)sizeof(ttl);
-		if(getsockopt(fd, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &ttl,&slen)<0) {
-			perror("get HOPLIMIT on socket");
-			return TTL_IGNORE;
-		}
-#endif
-	} else {
-#if !defined(IP_RECVTTL)
-		UNUSED_ARG(fd);
-		do { return TTL_IGNORE; } while(0);
-#else
-		socklen_t slen = (socklen_t)sizeof(ttl);
-		if(getsockopt(fd, IPPROTO_IP, IP_TTL, &ttl,&slen)<0) {
-			perror("get TTL on socket");
-			return TTL_IGNORE;
-		}
-#endif
-	}
-
-	CORRECT_RAW_TTL(ttl);
-
-	return ttl;
-}
-
-static int get_raw_socket_tos(evutil_socket_t fd, int family)
-{
-	int tos = 0;
-
-	if(family == AF_INET6) {
-#if !defined(IPV6_RECVTCLASS)
-		UNUSED_ARG(fd);
-		do { return TOS_IGNORE; } while(0);
-#else
-		socklen_t slen = (socklen_t)sizeof(tos);
-		if(getsockopt(fd, IPPROTO_IPV6, IPV6_TCLASS, &tos,&slen)<0) {
-			perror("get TCLASS on socket");
-			return -1;
-		}
-#endif
-	} else {
-#if !defined(IP_RECVTOS)
-		UNUSED_ARG(fd);
-		do { return TOS_IGNORE; } while(0);
-#else
-		socklen_t slen = (socklen_t)sizeof(tos);
-		if(getsockopt(fd, IPPROTO_IP, IP_TOS, &tos,&slen)<0) {
-			perror("get TOS on socket");
-			return -1;
-		}
-#endif
-	}
-
-	CORRECT_RAW_TOS(tos);
-
-	return tos;
-}
-
-static int set_raw_socket_ttl(evutil_socket_t fd, int family, int ttl)
-{
-
-	if(family == AF_INET6) {
-#if !defined(IPV6_RECVHOPLIMIT)
-		UNUSED_ARG(fd);
-		UNUSED_ARG(ttl);
-#else
-		CORRECT_RAW_TTL(ttl);
-		if(setsockopt(fd, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &ttl,sizeof(ttl))<0) {
-			perror("set HOPLIMIT on socket");
-			return -1;
-		}
-#endif
-	} else {
-#if !defined(IP_RECVTTL)
-		UNUSED_ARG(fd);
-		UNUSED_ARG(ttl);
-#else
-		CORRECT_RAW_TTL(ttl);
-		if(setsockopt(fd, IPPROTO_IP, IP_TTL, &ttl,sizeof(ttl))<0) {
-			perror("set TTL on socket");
-			return -1;
-		}
-#endif
-	}
-
-	return 0;
-}
-
-static int set_raw_socket_tos(evutil_socket_t fd, int family, int tos)
-{
-
-	if(family == AF_INET6) {
-#if !defined(IPV6_RECVTCLASS)
-		UNUSED_ARG(fd);
-		UNUSED_ARG(tos);
-#else
-		CORRECT_RAW_TOS(tos);
-		if(setsockopt(fd, IPPROTO_IPV6, IPV6_TCLASS, &tos,sizeof(tos))<0) {
-			perror("set TCLASS on socket");
-			return -1;
-		}
-#endif
-	} else {
-#if !defined(IPV6_RECVTOS)
-		UNUSED_ARG(fd);
-		UNUSED_ARG(tos);
-#else
-		if(setsockopt(fd, IPPROTO_IP, IP_TOS, &tos,sizeof(tos))<0) {
-			perror("set TOS on socket");
-			return -1;
-		}
-#endif
-	}
-
-	return 0;
-}
-
 static int set_socket_ttl(ioa_socket_handle s, int ttl)
 {
 	if(s->default_ttl < 0) //Unsupported
@@ -1039,9 +910,7 @@ static int bind_ioa_socket(ioa_socket_handle s, const ioa_addr* local_addr, int
 	if (s && s->fd >= 0 && s->e && local_addr) {
 
 		int res = addr_bind(s->fd, local_addr, reusable);
-		if (res < 0) {
-			TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "%s: cannot bind socket, sat=%d, st=%d, reusable=%d\n", __FUNCTION__, (int)s->sat, (int)s->st, reusable);
-		} else {
+		if (res >= 0) {
 			s->bound = 1;
 			addr_cpy(&(s->local_addr), local_addr);
 			if(addr_get_port(local_addr)<1) {
@@ -1131,7 +1000,6 @@ int create_relay_ioa_sockets(ioa_engine_handle e,
 					addr_set_port(&rtcp_local_addr, rtcp_port);
 					if (bind_ioa_socket(*rtcp_s, &rtcp_local_addr,
 						(transport == STUN_ATTRIBUTE_TRANSPORT_TCP_VALUE)) < 0) {
-						TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "%s: cannot bind rtcp socket, sat=%d, st=%d, transport=%d\n", __FUNCTION__, (int)(*rtcp_s)->sat, (int)(*rtcp_s)->st, transport);
 						addr_set_port(&local_addr, port);
 						turnipports_release(tp, transport, &local_addr);
 						turnipports_release(tp, transport, &rtcp_local_addr);
@@ -1177,7 +1045,6 @@ int create_relay_ioa_sockets(ioa_engine_handle e,
 					(transport == STUN_ATTRIBUTE_TRANSPORT_TCP_VALUE)) >= 0) {
 					break;
 				} else {
-					TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "%s: cannot bind rtp socket, sat=%d, st=%d, transport=%d\n", __FUNCTION__, (int)(*rtp_s)->sat, (int)(*rtp_s)->st, transport);
 					IOA_CLOSE_SOCKET(*rtp_s);
 					int rtcp_bound = 0;
 					if (rtcp_s && *rtcp_s) {
@@ -1355,7 +1222,6 @@ ioa_socket_handle ioa_create_connecting_tcp_relay_socket(ioa_socket_handle s, io
 #endif
 
 	if(bind_ioa_socket(ret, &new_local_addr,1)<0) {
-		TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR, "%s: cannot bind socket, sat=%d, st=%d\n", __FUNCTION__, (int)s->sat, (int)s->st);
 		IOA_CLOSE_SOCKET(ret);
 		ret = NULL;
 		goto ccs_end;
@@ -1373,7 +1239,7 @@ ioa_socket_handle ioa_create_connecting_tcp_relay_socket(ioa_socket_handle s, io
 
 	ret->conn_bev = bufferevent_socket_new(ret->e->event_base,
 					ret->fd,
-					BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+					TURN_BUFFEREVENTS_OPTIONS);
 	bufferevent_setcb(ret->conn_bev, NULL, NULL, connect_eventcb, ret);
 
 	ret->conn_arg = arg;
@@ -1457,6 +1323,7 @@ void add_socket_to_map(ioa_socket_handle s, ur_addr_map *amap)
 void delete_socket_from_map(ioa_socket_handle s)
 {
 	if(s && s->sockets_container) {
+
 		ur_addr_map_del(s->sockets_container,
 				&(s->remote_addr),
 				NULL);
@@ -1481,18 +1348,20 @@ ioa_socket_handle create_ioa_socket_from_fd(ioa_engine_handle e,
 	ret->magic = SOCKET_MAGIC;
 
 	ret->fd = fd;
-	ret->family = local_addr->ss.sa_family;
 	ret->st = st;
 	ret->sat = sat;
 	ret->e = e;
 
 	if (local_addr) {
+		ret->family = local_addr->ss.sa_family;
 		ret->bound = 1;
 		addr_cpy(&(ret->local_addr), local_addr);
 	}
 
 	if (remote_addr) {
 		ret->connected = 1;
+		if(!(ret->family))
+			ret->family = remote_addr->ss.sa_family;
 		addr_cpy(&(ret->remote_addr), remote_addr);
 	}
 
@@ -1533,6 +1402,7 @@ static void set_socket_ssl(ioa_socket_handle s, SSL *ssl)
 	if(s && (s->ssl != ssl)) {
 		if(s->ssl) {
 			SSL_set_app_data(s->ssl,NULL);
+			SSL_set_info_callback(s->ssl, (ssl_info_callback_t)NULL);
 		}
 		s->ssl = ssl;
 		if(ssl) {
@@ -1677,7 +1547,7 @@ void close_ioa_socket(ioa_socket_handle s)
 	}
 }
 
-ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
+ioa_socket_handle detach_ioa_socket(ioa_socket_handle s)
 {
 	ioa_socket_handle ret = NULL;
 
@@ -1709,13 +1579,31 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 
 		evutil_socket_t udp_fd = -1;
 
-		if(s->parent_s && full_detach) {
+		if(s->parent_s) {
 			udp_fd = socket(s->local_addr.ss.sa_family, SOCK_DGRAM, 0);
 			if (udp_fd < 0) {
 				perror("socket");
 				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"%s: Cannot allocate new socket\n",__FUNCTION__);
 				return ret;
 			}
+			if(sock_bind_to_device(udp_fd, (unsigned char*)(s->e->relay_ifname))<0) {
+				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"Cannot bind udp server socket to device %s\n",(char*)(s->e->relay_ifname));
+			}
+
+			if(addr_bind(udp_fd,&(s->local_addr),1)<0) {
+				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"Cannot bind new detached udp server socket to local addr\n");
+				close(udp_fd);
+				return ret;
+			}
+
+			int connect_err=0;
+			if(addr_connect(udp_fd, &(s->remote_addr), &connect_err)<0) {
+				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"Cannot connect new detached udp server socket to remote addr\n");
+				close(udp_fd);
+				return ret;
+			}
+			set_raw_socket_ttl_options(udp_fd, s->local_addr.ss.sa_family);
+			set_raw_socket_tos_options(udp_fd, s->local_addr.ss.sa_family);
 		}
 
 		detach_socket_net_data(s);
@@ -1737,12 +1625,14 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 
 		ret->magic = SOCKET_MAGIC;
 
-		ret->username_hash = s->username_hash;
+		SSL* ssl = s->ssl;
+		set_socket_ssl(s,NULL);
+		set_socket_ssl(ret,ssl);
 
-		set_socket_ssl(ret,s->ssl);
 		ret->fd = s->fd;
 
-		ret->family = s->family;
+		ret->family = get_ioa_socket_address_family(s);
+
 		ret->st = s->st;
 		ret->sat = s->sat;
 		ret->bound = s->bound;
@@ -1752,39 +1642,14 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 		addr_cpy(&(ret->remote_addr),&(s->remote_addr));
 
 		STRCPY(ret->orig_ctx_type, s->orig_ctx_type);
-
-		ioa_socket_handle parent_s = s->parent_s;
-		ur_addr_map *sockets_container = s->sockets_container;
 		
 		delete_socket_from_map(s);
 		delete_socket_from_parent(s);
 
-		if(udp_fd<0) {
-
-		  add_socket_to_parent(parent_s, ret);
-		  add_socket_to_map(ret,sockets_container);
-
-		} else {
+		if(udp_fd>=0) {
 
 			ret->fd = udp_fd;
 
-			if(sock_bind_to_device(udp_fd, (unsigned char*)(s->e->relay_ifname))<0) {
-			    TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"Cannot bind udp server socket to device %s\n",(char*)(s->e->relay_ifname));
-			}
-
-			if(addr_bind(udp_fd,&(s->local_addr),1)<0) {
-				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"Cannot bind new detached udp server socket to local addr\n");
-				IOA_CLOSE_SOCKET(ret);
-				return ret;
-			}
-
-			int connect_err=0;
-			if(addr_connect(udp_fd, &(s->remote_addr), &connect_err)<0) {
-				TURN_LOG_FUNC(TURN_LOG_LEVEL_ERROR,"Cannot connect new detached udp server socket to remote addr\n");
-				IOA_CLOSE_SOCKET(ret);
-				return ret;
-			}
-
 			set_socket_options(ret);
 		}
 
@@ -1794,7 +1659,6 @@ ioa_socket_handle detach_ioa_socket(ioa_socket_handle s, int full_detach)
 		ret->current_tos = s->current_tos;
 		ret->default_tos = s->default_tos;
 
-		set_socket_ssl(s,NULL);
 		s->fd = -1;
 	}
 
@@ -1835,10 +1699,15 @@ void set_ioa_socket_sub_session(ioa_socket_handle s, tcp_connection *tc)
 }
 
 int get_ioa_socket_address_family(ioa_socket_handle s) {
-	if(!s) {
+
+	int first_time = 1;
+	beg:
+	if (!(s && (s->magic == SOCKET_MAGIC) && !(s->done))) {
 		return AF_INET;
-	} else if(s->parent_s) {
-		return s->parent_s->family;
+	} else if(first_time && s->parent_s && (s != s->parent_s)) {
+		first_time = 0;
+		s = s->parent_s;
+		goto beg;
 	} else {
 		return s->family;
 	}
@@ -2278,13 +2147,11 @@ static TURN_TLS_TYPE check_tentative_tls(ioa_socket_raw fd)
 		if((s[0]==22)&&(s[1]==3)&&(s[5]==1)&&(s[9]==3)) {
 			char max_supported = (char)(TURN_TLS_TOTAL-2);
 			if(s[10] >= max_supported)
-				ret = (TURN_TLS_TYPE)((((int)TURN_TLS_TOTAL)-1));
+				ret = TURN_TLS_SSL23; /* compatibility mode */
 			else
 				ret = (TURN_TLS_TYPE)(s[10]+1);
 		} else if((s[2]==1)&&(s[3]==3)) {
 			ret = TURN_TLS_SSL23; /* compatibility mode */
-		} else if((s[2]==1)&&(s[3]==0)&&(s[4]==2)) {
-			ret = TURN_TLS_SSL23; /* old mode */
 		}
 	}
 
@@ -2407,7 +2274,7 @@ static int socket_input_worker(ioa_socket_handle s)
 								s->fd,
 								s->ssl,
 								BUFFEREVENT_SSL_ACCEPTING,
-								BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+								TURN_BUFFEREVENTS_OPTIONS);
 				bufferevent_setcb(s->bev, socket_input_handler_bev, socket_output_handler_bev,
 								eventcb_bev, s);
 				bufferevent_setwatermark(s->bev, EV_READ|EV_WRITE, 0, BUFFEREVENT_HIGH_WATERMARK);
@@ -2422,7 +2289,7 @@ static int socket_input_worker(ioa_socket_handle s)
 			}
 			s->bev = bufferevent_socket_new(s->e->event_base,
 							s->fd,
-							BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+							TURN_BUFFEREVENTS_OPTIONS);
 			bufferevent_setcb(s->bev, socket_input_handler_bev, socket_output_handler_bev,
 					eventcb_bev, s);
 			bufferevent_setwatermark(s->bev, EV_READ|EV_WRITE, 0, BUFFEREVENT_HIGH_WATERMARK);
@@ -3244,7 +3111,7 @@ int register_callback_on_ioa_socket(ioa_engine_handle e, ioa_socket_handle s, in
 					} else {
 						s->bev = bufferevent_socket_new(s->e->event_base,
 										s->fd,
-										BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+										TURN_BUFFEREVENTS_OPTIONS);
 						bufferevent_setcb(s->bev, socket_input_handler_bev, socket_output_handler_bev,
 							eventcb_bev, s);
 						bufferevent_setwatermark(s->bev, EV_READ|EV_WRITE, 0, BUFFEREVENT_HIGH_WATERMARK);
@@ -3268,13 +3135,13 @@ int register_callback_on_ioa_socket(ioa_engine_handle e, ioa_socket_handle s, in
 											s->fd,
 											s->ssl,
 											BUFFEREVENT_SSL_ACCEPTING,
-											BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+											TURN_BUFFEREVENTS_OPTIONS);
 						} else {
 							s->bev = bufferevent_openssl_socket_new(s->e->event_base,
 											s->fd,
 											s->ssl,
 											BUFFEREVENT_SSL_OPEN,
-											BEV_OPT_THREADSAFE | BEV_OPT_DEFER_CALLBACKS | BEV_OPT_UNLOCK_CALLBACKS);
+											TURN_BUFFEREVENTS_OPTIONS);
 						}
 						bufferevent_setcb(s->bev, socket_input_handler_bev, socket_output_handler_bev,
 							eventcb_bev, s);
@@ -3340,32 +3207,6 @@ void set_ioa_socket_tobeclosed(ioa_socket_handle s)
 		s->tobeclosed = 1;
 }
 
-static u32bits string_hash(const u08bits *str) {
-
-  u32bits hash = 0;
-  int c = 0;
-
-  while ((c = *str++))
-    hash = c + (hash << 6) + (hash << 16) - hash;
-
-  return hash;
-}
-
-int check_username_hash(ioa_socket_handle s, u08bits *username)
-{
-	if(s && username && username[0]) {
-		return (s->username_hash == string_hash(username));
-	}
-	return 1;
-}
-
-void set_username_hash(ioa_socket_handle s, u08bits *username)
-{
-	if(s && username && username[0]) {
-		s->username_hash = string_hash(username);
-	}
-}
-
 /*
  * Network buffer functions
  */
