@@ -129,7 +129,6 @@ static char Usage[] =
   "	-k	Private key file (for secure connections only).\n"
   "	-E	CA file for server certificate verification, \n"
   "		if the server certificate to be verified.\n"
-  "	-F	Cipher suite for TLS/DTLS. Default value is DEFAULT.\n"
   "	-p	TURN server port (Default: 3478 unsecure, 5349 secure).\n"
   "	-n	Number of messages to send (Default: 5).\n"
   "	-d	Local interface device (optional).\n"
@@ -306,6 +305,7 @@ int main(int argc, char **argv)
 		case 'P':
 			passive_tcp = 1;
 			/* implies 'T': */
+			/* no break */
 		case 'T':
 			relay_transport = STUN_ATTRIBUTE_TRANSPORT_TCP_VALUE;
 			break;
@@ -328,7 +328,7 @@ int main(int argc, char **argv)
 				exit(-1);
 			}
 			STRCPY(cert_file,fn);
-			turn_free(fn,strlen(fn)+1);
+			free(fn);
 		}
 			break;
 		case 'k':
@@ -339,7 +339,7 @@ int main(int argc, char **argv)
 				exit(-1);
 			}
 			STRCPY(pkey_file,fn);
-			turn_free(fn,strlen(fn)+1);
+			free(fn);
 		}
 			break;
 		default:
@@ -388,7 +388,7 @@ int main(int argc, char **argv)
 						g_upwd[pwd_length]=0;
 					}
 				}
-				turn_free(pwd,strlen(pwd)+1);
+				free(pwd);
 			}
 		}
 	}
@@ -437,18 +437,13 @@ int main(int argc, char **argv)
 		SSL_load_error_strings();
 		OpenSSL_add_ssl_algorithms();
 
-		const char *csuite = "ALL:SSLv2"; //"AES256-SHA" "DH"
+		const char *csuite = "ALL"; //"AES256-SHA" "DH"
 		if(use_null_cipher)
 			csuite = "eNULL";
 		else if(cipher_suite[0])
 			csuite=cipher_suite;
 
 		if(use_tcp) {
-#ifndef OPENSSL_NO_SSL2
-		  root_tls_ctx[root_tls_ctx_num] = SSL_CTX_new(SSLv2_client_method());
-		  SSL_CTX_set_cipher_list(root_tls_ctx[root_tls_ctx_num], csuite);
-		  root_tls_ctx_num++;
-#endif
 		  root_tls_ctx[root_tls_ctx_num] = SSL_CTX_new(SSLv23_client_method());
 		  SSL_CTX_set_cipher_list(root_tls_ctx[root_tls_ctx_num], csuite);
 		  root_tls_ctx_num++;
