@@ -31,7 +31,7 @@
 #ifndef __IOADEFS__
 #define __IOADEFS__
 
-#define TURN_SERVER_VERSION "3.2.4.5"
+#define TURN_SERVER_VERSION "3.2.5.5"
 #define TURN_SERVER_VERSION_NAME "Marshal West"
 #define TURN_SOFTWARE "Citrix-" TURN_SERVER_VERSION " '" TURN_SERVER_VERSION_NAME "'"
 
@@ -201,6 +201,10 @@ typedef enum _SHATYPE SHATYPE;
 
 #define SHA_TOO_WEAK (426)
 
+//////////////// Bufferevents /////////////////////
+
+#define TURN_BUFFEREVENTS_OPTIONS (BEV_OPT_DEFER_CALLBACKS | BEV_OPT_THREADSAFE | BEV_OPT_UNLOCK_CALLBACKS)
+
 //////////////// KERNEL-LEVEL CHANNEL HANDLERS /////////
 
 #if !defined(TURN_CHANNEL_HANDLER_KERNEL)
